<?php
// $Id: sunmailer.xml.inc,v 1.1.2.30 2010/04/30 19:35:56 mwacker Exp $

/**
 * @file
 * Builds the XML newsletter and the XSL to transform it.
 */

/**
 * Generates the XML for a newsletter with selected sections.
 */
function sunmailer_xml_newsletter($sids, $reset = FALSE) {
  static $cached = FALSE;
  static $sections = array();
  static $no_section_title = FALSE;
  // Build all sections once, then cache them.
  if (!$cached || $reset) {
    $cached = TRUE;
    // Impersonate an anonymous user.
    global $user;
    $original_user = $user;
    session_save_session(FALSE);
    $user = user_load(array('uid' => 0));
    // Build the sections.
    $sections = sunmailer_get_sections();
    views_get_all_views(); // http://drupal.org/node/670308
    foreach ($sections as &$sectionn) { // not a typo, see PHP's foreach() documentation
      sunmailer_get_section_nodes($sectionn);
    }
    if (count($sections) == 1) {
      $no_section_title = TRUE;
    }
    // Revert to original user.
    $user = $original_user;
    session_save_session(TRUE);
  }
  // Build all sections once, then cache them.
  $dom = new DOMDocument('1.0', variable_get('sunmailer_char_set', 'utf-8'));
  $newsletter_node = $dom->createElement('newsletter');
  $dom->appendChild($newsletter_node);
  sunmailer_xml_header($dom, $newsletter_node);
  $num_sections = 0;
  foreach ($sections as $sid => $section) {
    if (in_array($sid, $sids) && count($section['nodes'])) {
      $num_sections++;
      $newsletter_node->appendChild(sunmailer_xml_section($dom, $section, count($sections) == 1));
    }
  }
  if (!$num_sections) {
    return NULL;
  }
  sunmailer_xml_footer($dom, $newsletter_node);
  // Allow developers to alter the newsletter XML.
  module_invoke_all('sunmailer_xml_newsletter_alter', $newsletter_node, $dom);
  return $dom;
}

/**
 * Retrieves the nodes for an individual section.
 */
function sunmailer_get_section_nodes(&$section) {
  global $base_root;
  $view = views_get_view($section['view']);
  $view->execute_display($section['display'], $section['args'] ? explode('/', $section['args']) : array());
  $section['nodes'] = array();
  foreach ($view->result as $row_key => $row) {
    if (!$row->nid) {
      continue;
    }
    $node_fields = array();
    $node = node_load($row->nid);
    if ($node->created < variable_get('sunmailer_cutoff', 0)) {
      continue;
    }
    $node_fields['node'] = $node;
    foreach ($view->field as $field => $v) {
      $value = $view->render_field($field, $row_key);
      $value = preg_replace('|href="/|', "href=\"$base_root/", $value); // Make all URLs absolute.
      if ($value) {
        $node_fields[$field] = $value;
      }
    }
    $section['nodes'][] = $node_fields;
  }
  return $section;
}

/**
 * Adds an XML element with its data enclosed in a CDATA section.
 */
function _sunmailer_append_cdata_child(&$dom, &$parent_node, $title, $data) {
  $child_node = $dom->createElement($title);
  $child_node->appendChild($dom->createCDATASection($data));
  $parent_node->appendChild($child_node);
}

/**
 * Generates the XML for the header.
 */
function sunmailer_xml_header(&$dom, &$newsletter_node) {
  // Retrieve the header information.
  $encoding = variable_get('sunmailer_char_set', 'utf-8');
  $title = htmlspecialchars_decode(filter_xss(variable_get('sunmailer_title', variable_get('site_name', t('SunMailer')))));
  if ($logo_fid = variable_get('sunmailer_logo_fid', 0)) {
    $path = db_result(db_query("SELECT filepath FROM {files} WHERE fid = %d", $logo_fid));
    $logo = url($path, array('absolute' => TRUE));
  }
  $link = url('<front>', array('absolute' => TRUE));
  $date_format = variable_get('sunmailer_date_format', 'l, F j, Y');
  if ($date_format) {
    $date = filter_xss(date($date_format));
  }
  $message = filter_xss(variable_get('sunmailer_message', ''));
  // Build the header XML.
  _sunmailer_append_cdata_child($dom, $newsletter_node, 'title', $title);
  if ($logo) {
    _sunmailer_append_cdata_child($dom, $newsletter_node, 'logo', $logo);
  }
  _sunmailer_append_cdata_child($dom, $newsletter_node, 'link', $link);
  if ($date) {
    _sunmailer_append_cdata_child($dom, $newsletter_node, 'date', $date);
  }
  if ($message) {
    _sunmailer_append_cdata_child($dom, $newsletter_node, 'message', $message);
  }
  return;
}

/**
 * Generates the XML for a section.
 */
function sunmailer_xml_section(&$dom, $section, $no_title) {
  // Return nothing if the section has no content.
  if (!count($section['nodes'])) {
    return '';
  }
  // Build the section XML.
  $section_node = $dom->createElement('section');
  if (!$no_title) {
    _sunmailer_append_cdata_child($dom, $section_node, 'name', htmlspecialchars_decode(filter_xss($section['name'])));
  }
  foreach ($section['nodes'] as $node_fields) {
    $section_node->appendChild(sunmailer_xml_content($dom, $node_fields));
  }
  // Allow developers to alter the section XML.
  module_invoke_all('sunmailer_xml_section_alter', $section_node, $dom, $section);
  return $section_node;
}

/**
 * Generates the XML for a content piece.
 */
function sunmailer_xml_content(&$dom, $node_fields) {
  // Retrieve the content information.
  $node = $node_fields['node'];
  $title = filter_xss($node->title);
  $link = url("node/{$node->nid}", array('absolute' => TRUE));
  if ($node->uid) {
    $author = filter_xss($node->name);
    $authorlink = url("user/{$node->uid}", array('absolute' => TRUE));
  }
  $date = format_date($node->created);
  // Add basic node information to the content XML.
  $content_node = $dom->createElement('content');
  _sunmailer_append_cdata_child($dom, $content_node, 'title', $title);
  _sunmailer_append_cdata_child($dom, $content_node, 'link', $link);
  if ($author) {
    _sunmailer_append_cdata_child($dom, $content_node, 'author', $author);
    _sunmailer_append_cdata_child($dom, $content_node, 'authorlink', $authorlink);
  }
  _sunmailer_append_cdata_child($dom, $content_node, 'date', $date);
  _sunmailer_append_cdata_child($dom, $content_node, 'teaser', $node->teaser);
  // Add additional fields to the content XML.
  foreach ($node_fields as $name => $field) {
    if ($name == 'node') {
      continue;
    }
    _sunmailer_append_cdata_child($dom, $content_node, $name, $field);
  }
  // Allow developers to alter the content XML.
  module_invoke_all('sunmailer_xml_content_alter', $content_node, $dom, $node);
  return $content_node;
}

/**
 * Generates the XML for the footer.
 */
function sunmailer_xml_footer(&$dom, &$newsletter_node) {
  // Retrieve the footer information.
  $unsubscribe = url('sunmailer/unsubscribe', array('absolute' => TRUE));
  $password = url('user/password', array('absolute' => TRUE));
  $notice = t('To unsubscribe, go to <a href="!unsubscribe">!unsubscribe</a>. ', array('!unsubscribe' => $unsubscribe));
  $notice .= t('Forgot your password? Reset it at <a href="!password">!password</a>.', array('!password' => $password));
  // Build the footer XML.
  _sunmailer_append_cdata_child($dom, $newsletter_node, 'notice', $notice);
  return;
}

/**
 * Transforms XML into text or HTML.
 */
function sunmailer_transform_xml($xml, $format) {
  // Load XSL.
  $path = db_result(db_query("SELECT filepath FROM {files} WHERE fid = %d", $format['fid']));
  $doc = new DOMDocument();
  $doc->load($path);
  $xslt = new XSLTProcessor();
  $xslt->importStyleSheet($doc);
  // Transform XML, do some extra processing for text formats.
  $newsletter = $xslt->transformToXML($xml);
  if (!$format['html']) {
    $newsletter = strip_tags($newsletter);
    $newsletter = html_entity_decode($newsletter);
  }
  return $newsletter;
}